@@ -2,7 +2,7 @@ use {
     crate::{
         format::CodeStr,
         state::{self, State},
-        Settings,
+        Settings, Threshold,
     },
     byte_unit::Byte,
     chrono::DateTime,
@@ -20,6 +20,12 @@ use {
     },
 };
 
+#[cfg(target_os = "linux")]
+use {
+    std::path::{Path, PathBuf},
+    sysinfo::{Disk, DiskExt, RefreshKind, System, SystemExt},
+};
+
 // When querying Docker for the image IDs corresponding to a list of container IDs, this is the
 // maximum number of container IDs to query at once.
 const CONTAINER_IDS_CHUNK_SIZE: usize = 100;
@@ -218,7 +224,7 @@ fn list_image_records(state: &mut State) -> io::Result<HashMap<String, ImageReco
 
 // Ask Docker for the IDs of the images currently in use by containers.
 fn image_ids_in_use() -> io::Result<HashSet<String>> {
-    // Query Docker for the container IDs.
+    // Query Docker for all the container IDs.
     let container_ids_output = Command::new("docker")
         .args(&[
             "container",
@@ -303,6 +309,57 @@ fn image_ids_in_use() -> io::Result<HashSet<String>> {
     Ok(image_ids)
 }
 
+// Determine Docker's root directory.
+#[cfg(target_os = "linux")]
+fn docker_root_dir() -> io::Result<PathBuf> {
+    // Query Docker for it.
+    let output = Command::new("docker")
+        .args(&["info", "--format", "{{.DockerRootDir}}"])
+        .stderr(Stdio::inherit())
+        .output()?;
+
+    // Ensure the command succeeded.
+    if !output.status.success() {
+        return Err(io::Error::new(
+            io::ErrorKind::Other,
+            "Unable to determine the Docker root directory.",
+        ));
+    }
+
+    // Trim the output.
+    String::from_utf8(output.stdout)
+        .map(|s| PathBuf::from(s.trim()))
+        .map_err(|error| io::Error::new(io::ErrorKind::Other, error))
+}
+
+// Find the disk containing a path.
+#[cfg(target_os = "linux")]
+fn get_disk_by_file<'a>(disks: &'a [Disk], path: &Path) -> io::Result<&'a Disk> {
+    disks
+        .iter()
+        .filter(|d| path.starts_with(d.mount_point()))
+        .max_by_key(|d| d.mount_point().as_os_str().len())
+        .ok_or_else(|| {
+            io::Error::new(
+                io::ErrorKind::Other,
+                format!(
+                    "Unable to find disk for path {}.",
+                    path.to_string_lossy().code_str(),
+                ),
+            )
+        })
+}
+
+// Find size of filesystem on which docker root directory is stored.
+#[cfg(target_os = "linux")]
+fn docker_root_dir_filesystem_size() -> io::Result<Byte> {
+    let root_dir = docker_root_dir()?;
+    let system = System::new_with_specifics(RefreshKind::new().with_disks_list());
+    let disks = system.disks();
+    let disk = get_disk_by_file(disks, &root_dir)?;
+    Ok(Byte::from(disk.total_space()))
+}
+
 // Get the total space used by Docker images.
 #[allow(clippy::map_err_ignore)]
 fn space_usage() -> io::Result<Byte> {
@@ -673,12 +730,28 @@ fn vacuum(
 
 // Stream Docker events and vacuum when necessary.
 pub fn run(settings: &Settings, state: &mut State, first_run: &mut bool) -> io::Result<()> {
+    // Determine the threshold in bytes.
+    let threshold = match settings.threshold {
+        Threshold::Absolute(b) => b,
+
+        #[cfg(target_os = "linux")]
+        Threshold::Percentage(p) =>
+        {
+            #[allow(
+                clippy::cast_precision_loss,
+                clippy::cast_possible_truncation,
+                clippy::cast_sign_loss
+            )]
+            Byte::from_bytes((p * docker_root_dir_filesystem_size()?.get_bytes() as f64) as u128)
+        }
+    };
+
     // NOTE: Don't change this log line, since the test in the Homebrew formula
     // (https://github.com/Homebrew/homebrew-core/blob/HEAD/Formula/docuum.rb) relies on it.
     info!("Performing an initial vacuum on startup\u{2026}");
 
     // Run the main vacuum logic.
-    vacuum(state, *first_run, settings.threshold, &settings.keep)?;
+    vacuum(state, *first_run, threshold, &settings.keep)?;
     state::save(state)?;
     *first_run = false;
 
@@ -755,7 +828,7 @@ pub fn run(settings: &Settings, state: &mut State, first_run: &mut bool) -> io::
         touch_image(state, &image_id, true)?;
 
         // Run the main vacuum logic.
-        vacuum(state, *first_run, settings.threshold, &settings.keep)?;
+        vacuum(state, *first_run, threshold, &settings.keep)?;
 
         // Persist the state.
         state::save(state)?;
